package eu.interedition.text.util;

import com.google.common.base.Throwables;
import com.google.common.io.Closeables;
import java.io.FilterReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * @author <a href="http://gregor.middell.net/" title="Homepage">Gregor Middell</a>
 */
class TextDigestingFilterReader extends FilterReader {
    public static final byte[] NULL_DIGEST = digest("");

    private MessageDigest digest;
    private byte[] result;
    private CharsetEncoder encoder;

    private TextDigestingFilterReader(Reader in) {
        super(in);
        try {
            this.digest = MessageDigest.getInstance("SHA-512");
            this.encoder = Charset.forName("UTF-8").newEncoder();
        } catch (NoSuchAlgorithmException e) {
            throw Throwables.propagate(e);
        }
    }

    @Override
    public int read() throws IOException {
        final int read = super.read();
        if (read >= 0) {
            digest.update(encoder.encode(CharBuffer.wrap(new char[]{(char) read})));
        }
        return read;
    }

    @Override
    public int read(char[] cbuf, int off, int len) throws IOException {
        final int read = super.read(cbuf, off, len);
        if (read >= 0) {
            digest.update(encoder.encode(CharBuffer.wrap(cbuf, off, read)));
        }
        return read;
    }

    @Override
    public void reset() throws IOException {
        digest.reset();
        result = null;
        super.reset();
    }

    byte[] digest() {
        if (result == null) {
            result = digest.digest();
            digest.reset();
        }
        return result;
    }

    private static byte[] digest(String str) {
        final TextDigestingFilterReader reader = new TextDigestingFilterReader(new StringReader(str));
        try {
            while (reader.read() >= 0) {
            }
            return reader.digest();
        } catch (IOException e) {
            throw Throwables.propagate(e);
        } finally {
            Closeables.closeQuietly(reader);
        }
    }
}
